import geopandas
import json
from pyproj import Transformer
import boto3

dynamodb = boto3.client('dynamodb')
from botocore.exceptions import ClientError

#convert shapefile to geojson
def shp_2_geojson(shapefile, geojson):
    myshpfile = geopandas.read_file(shapefile)
    myshpfile.to_file(geojson, driver='GeoJSON')

#upload geojson to the dynamo database specifically for lines.geojson
def lines_2_dynamo():
    with open("PCTA/lines.geojson") as f:
        data = json.load(f)
    data = data["features"]
    familiar_ids = []
    for item in data:
        id = item["properties"]["RecordID"]
        geometry = item["geometry"]
        if(geometry == None):
            points = []
        else:
            points=geometry["coordinates"]
        #add points to previous entry if id is familiar
        if id in familiar_ids:
            result = dynamodb.get_item(
                TableName="lines",
                Key={
                    'ID': {'S': id}
                },
                AttributesToGet=["points"]
            )
            newpoints=json.loads(result['Item']['points']["S"])
            newpoints.append(points)
            dynamodb.put_item(
                TableName='lines', 
                Item={
                    'ID':{'S':id},
                    'points':{'S': json.dumps(newpoints)}
                }
            )
        else:
            print(".")
            #otherwise create new entry
            familiar_ids.append(id)
            dynamodb.put_item(
                TableName='lines', 
                Item={
                    'ID':{'S':id},
                    'points':{'S': json.dumps([points])}
                }
            )

def projects_2_dynamo():
    with open("PCTA/projects.geojson",encoding="utf8") as f:
        data = json.load(f)
    data = data["features"]
    familiar_ids = []
    for item in data:
        properties = item["properties"]
        projectStatus=properties["Project_St"]
        if(projectStatus != "Cancelled"):
            id = properties["Id_15"]
            if id in familiar_ids:
                print("familiar")
                return
            familiar_ids.append(id)
            projectDate=properties["Project_Da"]
            startMonth=properties["Start_Mont"]
            pctLength=str(properties["PCT_Length"])
            sideTrail=str(properties["Side_Trail"])
            milesOfTrail=str(properties["Miles_of_T"])
            print(len(familiar_ids))
            dynamodb.put_item(
                TableName='projects', 
                Item={
                    'ID':{'S':id},
                    'date':{'S':projectDate},
                    'month':{'S':startMonth},
                    'pctLength':{'N':pctLength},
                    'sideLength':{'N':sideTrail},
                    'miles':{'N':milesOfTrail}
                }
            )
def duplicates():
    with open("PCTA/projects.geojson",encoding="utf8") as f:
        data = json.load(f)
    data = data["features"]
    familiar_ids = []
    for item in data:
        properties = item["properties"]
        projectStatus=properties["Project_St"]
        if(projectStatus != "Cancelled"):
            id = properties["Id_15"].strip()
            print(len(familiar_ids))
            if id in familiar_ids:
                print("familiar")
            familiar_ids.append(id)
    with open("dump.json", 'w', encoding="utf8") as f:
        json.dump(familiar_ids, f)

def add_field(file,table, key, tableKey):
    with open(file,encoding="utf8") as f:
        data = json.load(f)
    data = data["features"]
    for item in data:
        properties = item["properties"]
        projectStatus=properties["Project_St"]
        if(projectStatus != "Cancelled"):
            id = properties["Id_15"]
            value = properties[key]
            print(id)
            print(value)
            try:
                # Update the item in the table
                response = dynamodb.update_item(
                    TableName=table,
                    Key={
                        'ID': {'S': str(id)}  # Assuming 'id' is the partition key, adjust if necessary
                    },
                    UpdateExpression=f'SET {str(tableKey)} = :value',
                    ExpressionAttributeValues={
                        ':value': {'N': str(value)}  # Adjust the data type (e.g., S for string)
                    },
                    ReturnValues='ALL_NEW'  # Optional: return the updated item
                )
            except ClientError as e:
                print(f"Error updating item: {e.response['Error']['Message']}")

        
def ids_2_json():
    with open("PCTA/lines.geojson") as f:
        data = json.load(f)
    data = data["features"]
    familiar_ids = []
    for item in data:
        id = item["properties"]["RecordID"]
        #add points to previous entry if id is familiar
        if not id in familiar_ids:
            print(".")
            #otherwise create new entry
            familiar_ids.append(id)
    with open("PCTA/ids.json", "w") as f:
        json.dump(familiar_ids, f)
    print(len(familiar_ids))

def mark_lines():
    with open("key_files/ids.json") as f:
        ids = json.load(f)
    for id in ids:
        dynamodb.update_item(
            TableName='projects',
            Key={'ID':{'S':id}},
            UpdateExpression=
                "SET #attr = :val"
            ,
            ExpressionAttributeNames={'#attr': "line"},
            ExpressionAttributeValues={':val': {"BOOL":True}}
        )
#puts lat lon values for all the points currently in EPSG:3857        
def convert_points():
    with open("key_files/points.json") as f:
        points = json.load(f)
    for index,point in enumerate(points):
        strpoint = json.dumps(point)
        transformer = Transformer.from_crs("EPSG:3857", "EPSG:4326")
        lat,lon = transformer.transform(point[0],point[1])
        latlon = json.dumps([lat,lon])
        print(index)
        dynamodb.update_item(
            TableName='points',
            Key={'point':{'S':strpoint}},
            UpdateExpression=
                "SET #attr = :val"
            ,
            ExpressionAttributeNames={'#attr': "latlon"},
            ExpressionAttributeValues={':val': {"S":latlon}}
        )


#averages the years for 2022,2023,2024
def get_averages(points = "points.json"):
    with open(f"key_files/{points}") as f:
        points_list = json.load(f)
    length = len(points_list)
    for n, point_array in enumerate(points_list):
        # get the point as a string and index into the database to get yearly values of work
        point_str = json.dumps(point_array)
        point = dynamodb.get_item(
            TableName="points",
            Key={
                'point': {'S': point_str}
            },
            AttributesToGet=["2022", "2023", "2024"]
        )
        attributes = point['Item']
        #returns 0 if it doesn't exist
        v2022 = float(json.loads(attributes.get('2022', {}).get('N', '0')))
        v2023 = float(json.loads(attributes.get('2023', {}).get('N', '0')))
        v2024 = float(json.loads(attributes.get('2024', {}).get('N', '0')))
        avg = (v2022+v2023+v2024)/3

        # update the item with avg
        dynamodb.update_item(
            TableName='points',
            Key={'point':{'S':point_str}},
            UpdateExpression=
                "SET #attr = :val"
            ,
            ExpressionAttributeNames={'#attr': "avg"},
            ExpressionAttributeValues={':val': {"N":str(avg)}}
        )

        #print progress
        print(f"{n}/{length}")

get_averages("1000point.json")