import matplotlib.pyplot as plt
import boto3
import json
from matplotlib.ticker import MaxNLocator
import numpy as np

dynamodb = boto3.client('dynamodb')

def graph_attributes(title, ind_str, ind_type, ind_title, dep_str, dep_type, dep_title, points = "points.json", is_dict = True):
    # initialize arrays of x and y vals
    x_vals = []
    y_vals = []
    if is_dict:
        # load everything
        with open(f"correlated/{points}") as f:
            points_list = json.load(f)
    else:
        # load points from file containing some subset of points
        with open(f"key_files/{points}") as f:
            points_list = json.load(f)

    length = len(points_list)

    # loop through the list of points
    for n, point_array in enumerate(points_list):
        """speeds up code for testing purposes"""
        """if not n % 10 == 0:
            continue"""
        if not is_dict:
            # get the point as a string and index into the database to get values
            point_str = json.dumps(point_array)
            point = dynamodb.get_item(
                TableName="points",
                Key={
                    'point': {'S': point_str}
                },
                AttributesToGet=[ind_str,dep_str]
            )
            attributes = point['Item']

        #get the attributes and append them to lists for graphing
        ind = (point_array[ind_str] if is_dict else attributes[ind_str][ind_type])
        dep = (point_array[dep_str] if is_dict else attributes[dep_str][dep_type])
        x_vals.append(float(ind))
        y_vals.append(float(dep))
    
        print(f"{n}/{length} \r", end="")
    print()

    #make best fit line
    x = np.array(x_vals)
    y = np.array(y_vals)
    a, b = np.polyfit(x, y, 1)
    print(a,b)

    # Create the line of best fit equation
    plt.plot(x, a*x+b)    # create the scatter plot
    plt.scatter(x_vals, y_vals, color='blue', s=0.2)
    plt.xlabel(ind_title)
    plt.ylabel(dep_title)
    plt.title(title)

    # Set axis limits to provide some spacing around the data points for readability
    x_padding = (max(x_vals) - min(x_vals)) * 0.1  # 10% padding
    y_padding = (max(y_vals) - min(y_vals)) * 0.1  # 10% padding
    
    plt.xlim(min(x_vals) - x_padding, max(x_vals) + x_padding)
    plt.ylim(min(y_vals) - y_padding, max(y_vals) + y_padding)
    
    # Adjust the number of ticks for readability using MaxNLocator
    plt.gca().xaxis.set_major_locator(MaxNLocator(integer=True))  # Make sure x-axis ticks are integers
    plt.gca().yaxis.set_major_locator(MaxNLocator(integer=True))  # Make sure y-axis ticks are integers

    # Make the layout tight to avoid clipping of labels
    plt.tight_layout()

    plt.grid(False)
    plt.show()

graph_attributes("Average Yearly Maintenance by Yearly Precipitation", "weather", "S", "Precipitation (mm)", "avg", "N", "Maintenance (Person Hours)","1000point.json")
#graph_attributes("Average Yearly Maintenance by Traffic", "traffic", "S", "Traffic (Relative Unit)", "avg", "N", "Maintenance (Person Hours)","1000point.json")
#graph_attributes("Average Yearly Maintenance by Slope", "slope", "S", "Slope (Degrees)", "avg", "N", "Maintenance (Person Hours)","1000point.json")