'''
This finds the slope of the terrain around a certain point
Points are imputed in coordinate format EPSG:3857 
This currently uses an example point but we should be able to load points from 1000point.json and use them easily
Still returns values that seem low for slope.
'''



import requests
from threading import Thread
from pyproj import Transformer
import math
#offset the points
offset = 100
#example point
lat,lon = [-11777735.709317284, 4272628.64473215]

# function to take one point and return the elevation
def process_point(item):
    #convert item to correct coordinate system
    transformer = Transformer.from_crs("EPSG:3857", "WGS84")
    coordinate = transformer.transform(item[0],item[1])

    #Make Api Request
    url = f"https://api.open-meteo.com/v1/elevation?latitude={coordinate[0]}&longitude={coordinate[1]}"
    response = requests.get(url)
    data = response.json()
    if "elevation" in data:
        elevation = data["elevation"][0]
    else:
        print(data)
        return False
    return elevation

def process_points(points, elevations=None):
    #puts elevations into list
    if elevations is None:
        elevations = []
    for point in points:
        elevations.append(process_point(point))
    return elevations

def get_elevations(points):
    length = len(points)
    elevations = []
    threads = []
    # create the threads
    for i in range(length):
        pointlist = points[i::length]
        t = Thread(target=process_points, args=(pointlist,elevations))
        threads.append(t)

    # start the threads
    [ t.start() for t in threads ]
    # wait for the threads to finish
    [ t.join() for t in threads ]
    return elevations

def find_slope(lat,lon):
    point = [lat,lon]
    #Get all offset points in a circle around the point (also put the point in incase its a peak)
    surrounding_points = []
    surrounding_points.append ((point[0], point[1]+offset))
    surrounding_points.append ((point[0]+(0.70710678118) * offset,point[1]+(0.70710678118) * offset))
    surrounding_points.append ((point[0]+offset,point[1]))
    surrounding_points.append ((point[0]+(0.70710678118) * offset,point[1]-(0.70710678118) * offset))
    surrounding_points.append ((point[0],point[1]-offset))
    surrounding_points.append ((point[0]-(0.70710678118) * offset,point[1]-(0.70710678118) * offset))
    surrounding_points.append ((point[0]-offset,point[1]))
    surrounding_points.append ((point[0]-(0.70710678118) * offset,point[1]+(0.70710678118) * offset))

    elevations = get_elevations(surrounding_points)

    #find slope based on max and min elevations
    elevation_difference = (max(elevations)-min(elevations))
    slope = elevation_difference/(offset*2)
    angle_rad = math.atan(slope)  # Calculate arctan in radians
    angle_deg = math.degrees(angle_rad)  # Convert radians to degrees
    
    return (angle_deg)




    
