from pyppeteer import *
import asyncio
from PIL import Image
import math
import boto3
import geopandas
from pyproj import Transformer
import logging
import os
import glob
import json

logging.getLogger('pyppeteer').setLevel(logging.WARNING)
logging.getLogger('websockets').setLevel(logging.WARNING)
logging.getLogger('PIL').setLevel(logging.WARNING)
logging.getLogger('asyncio').setLevel(logging.WARNING)

#Funcion that enters the coordinate and returns luminocity value
async def enter_coordinate(thread, page, mercators,index):
    #convert item to correct coordinate system
    transformer = Transformer.from_crs("EPSG:3857", "WGS84")
    lat,lon = transformer.transform(mercators[0],mercators[1])
    # Insert Coordinate
    if await page.querySelector('button[title="Clear"].btn-secondary'):
        await page.click('button[title="Clear"].btn-secondary')
    await page.type('input.fm-search-input.form-control', str(lat)+','+str(lon))
    await page.keyboard.press('Enter')
    await page.waitForSelector('div.flex-grow-1.text-truncate')
    await page.click('div.flex-grow-1.text-truncate')
    #print("coordinate entered")
    for i in range(5):
        await page.waitForSelector('button.btn.btn-secondary[title="Zoom in"]')
        await page.click('button.btn.btn-secondary[title="Zoom in"]')
        await asyncio.sleep(0.5)
    await asyncio.sleep(1.5)
    #print("screenshot taken")

    #Manipulate Screenshot
    traffic = 14.6
    while traffic > 14.5:
        luminocity_of_image = 0
        await page.screenshot(path=f"trail_use/screenshots/test_screenshot{thread}.png")
        img = Image.open(f"trail_use/screenshots/test_screenshot{thread}.png")
        width,height = img.size
        img = img.crop((width//5,height//5,4*(width//5),4*(height//5)))
        img.save(f"trail_use/screenshots/test_screenshot{thread}.png")
        img = img.convert("RGB")
        datas = img.getdata()
        #Change pixel colors
        count = 0
        for item in datas:
            if not item[1] > item[2]:
                count+=1
                brightness  =  math.sqrt(.241*(item[0]^2) + .691*(item[1]^2) + .068*(item[2]^2))
                luminocity_of_image = luminocity_of_image+brightness
        #print(index)

        traffic = luminocity_of_image/count if count!=0 else 0
    print(index, traffic)
    return(traffic)
    


#Function opens freemap sk, gets rid of popups, and calls function to enter coordinate
async def trail_use_scraper(latlongs):

    #delete and recreate the screenshots directory for ease of tracking loading progress
    files = glob.glob(os.path.join("trail_use/screenshots", '*'))
    for file in files:
        os.remove(file)

    #number of concurrent instances of chromium to run
    threads = 1
    pages = []
    #print("Running Login")
    browser = await launch(headless=True)
    #create all the pages to run each search
    print("loading")
    for i in range(threads):
        page = await browser.newPage()
        await page.setViewport({ 
            "width": 1200, 
            'height': 1200,
        })
        await page.goto("https://www.freemap.sk/#map=15/47.955904/-120.027030&layers=Cs0")
        await page.screenshot(path=f"trail_use/screenshots/test_screenshot{i}.png")
        #Get rid of all the popups
        for i in range(4):
            await page.waitForSelector('button.btn-close', visible=True)
            await page.click('button.btn-close')
        pages.append(page)
    print('loaded')
        
        
    #get the length for printining purposes
    length = len(latlongs)

    #group latlongs into groups of length "threads"
    latlongs_grouped = [latlongs[i:i + threads] for i in range(0, len(latlongs), threads)]
    tasks = [enter_coordinate(n,p,latlongs_grouped[0][n],0*threads+n) for n,p in enumerate(pages)]
    results = await asyncio.gather(*tasks)
    # For each group run it on each latlon
    all_traffics = {}
    for j, latlong_group in enumerate(latlongs_grouped):
        tasks = [enter_coordinate(n,p,latlong_group[n],j*threads+n) for n,p in enumerate(pages)]
        results = await asyncio.gather(*tasks)
        traffics = dict(zip([str(latlon) for latlon in latlong_group],results))
        all_traffics.update(traffics)
        
    await browser.close()
    with open(f"trail_use/1000traffics.json", 'w') as f:
        json.dump(all_traffics,f)

"""with open(f"key_files/1000point.json") as f:
    points_list = json.load(f)

asyncio.run(trail_use_scraper(points_list))"""

async def scrape_one(lat,lon):
    #delete and recreate the screenshots directory for ease of tracking loading progress
    files = glob.glob(os.path.join("trail_use/screenshots", '*'))
    for file in files:
        os.remove(file)

    #print("Running Login")
    browser = await launch(headless=True)
    #create all the pages to run each search
    print("loading")
    page = await browser.newPage()
    await page.setViewport({ 
        "width": 1200, 
        'height': 1200,
    })
    await page.goto("https://www.freemap.sk/#map=15/47.955904/-120.027030&layers=Cs0")
    await page.screenshot(path=f"trail_use/screenshots/test_screenshot0.png")
    #Get rid of all the popups
    for i in range(4):
        await page.waitForSelector('button.btn-close', visible=True)
        await page.click('button.btn-close')
    print('loaded')

    value = await enter_coordinate(0,page,[lat,lon],0)
    value = await enter_coordinate(0,page,[lat,lon],0)

    await browser.close()

    return value

        
#test(trail_use_scraper, "100latlon.json")