import requests
from timezonefinder import TimezoneFinder
from datetime import datetime, timedelta, UTC

#creates a best guess average annual rainfall based off the data available. 
def get_average_rainfall(lat, lon):
    # Fstart and end dates for collecting. This can change, and is limited by the api's history
    end_date_str = "2025-03-17"
    start_date_str = "2025-01-25"
    
    # Open-Meteo Historical Weather API URL
    url = "https://api.open-meteo.com/v1/forecast"
    
    # API parameters: Latitude, Longitude, Start and End date for historical data, and required hourly or daily data
    params = {
        'latitude': lat,
        'longitude': lon,
        'start_date': start_date_str,
        'end_date': end_date_str,
        'daily': 'precipitation_sum',  # Get daily total precipitation (rainfall)
        'timezone': 'UTC'  # Set timezone to UTC
    }
    
    # Make the API request
    response = requests.get(url, params=params)
    print(response)
    # Check if the request was successful
    if response.status_code != 200:
        raise Exception(f"Error fetching data from Open-Meteo API: {response.status_code}")
    
    # Parse the JSON response
    data = response.json()
    
    # Extract daily precipitation values
    daily_precipitation = data.get('daily', {}).get('precipitation_sum', [])
    
    if len(daily_precipitation) == 0:
        raise ValueError("No data")
    
    # Calculate the average rainfall for the days
    average_rainfall = sum(daily_precipitation) / len(daily_precipitation)
    
    return average_rainfall*365



"""point1 = [48.355115177425176,-120.88125721837294]
result1 = get_average_rainfall(*point1)
print(result1)
point1 = [35.6894,-105.9382]
result1 = get_average_rainfall(*point1)
print(result1)
point1 = [47.9504,-124.3855]
result1 = get_average_rainfall(*point1)
print(result1)"""